use strict;
package POE::Component::Server::HTTP::Connection;

sub new {
    return bless {};
}

sub remote_host {
    return "N/A";
}

sub remote_ip {
    my $self = shift;
    return $self->{remote_ip};
}

sub local_addr {
    my $self = shift;
    return $self->{local_addr};
}

sub remote_addr {
    my $self = shift;
    return $self->{remote_addr};
}

sub remote_logname {
    my $self = shift;
    return "N/A";
}

sub user {
    my $self = shift;
    if (@_) {
        $self->{user} = shift;
    }
    return $self->{user};
}

sub authtype {
    my $self = shift;
    return $self->{authtype};
}

sub aborted {
    return 0;
}

sub fileno {
    return 0;
}

sub clone {
    my $self = shift;
    my $new = bless { %$self };
    return $new;
}

sub response {
    my $self = shift;
    return $self->{response};
}

sub request {
    my $self = shift;
    return $self->{request};
}

sub ID {
    my $self = shift;
    return $self->{ID};
}

sub on_close
{
    my( $self, @data ) = @_;
    if( @data ) {
        $self->{OnClose} = [ $POE::Kernel::poe_kernel->get_active_session->ID,
                             @data
                           ];
    }
    else {
        delete $self->{OnClose};
    }
}

sub DESTROY {
    my( $self ) = @_;
    if( $self->{OnClose} ) {
        # warn "DESTROY ", join ', ', @{ $self->{OnClose} };
        $POE::Kernel::poe_kernel->call( @{ delete $self->{OnClose} } );
    }
}


1;

__END__

=head1 NAME

POE::Component::Server::HTTP::Connection - Connection object for a POE HTTP Daemon

=head1 SYNOPSIS

    sub handler1 {
        my ($request, $response) = @_;
        my $connect = $response->connection;
        my $uniqueID = $connect->ID;
        $connection->on_close( 'conn_close', $uniqueID );
    }

=head1 DESCRIPTION

POE::Component::Server::HTTP::Connection encapsulates information related to
a single connection.  Because of HTTP connection persistance, a single
connection may be use for more then one request and response.

B<DO NOT> keep a reference to the connection object.

=head1 METHODS

=head2 ID

Returns the connections unique identifier.

=head2 on_close

    $connection->on_close( $event, @args );

Register an event in the current session to be called when the connection
closes.  C<$event> is the event's name, C<@args> are pased as C<ARG0..$#_>.

=head2 remote_ip

=head2 remote_addr

IP address of the HTTP client.

=head2 local_addr

IP address the HTTP client connected to.

=head2 response

Get the current response object L<POE::Component::Server::HTTP::Response>.

=head2 request

Get the current request object L<POE::Component::Server::HTTP::Request>.

=head2 user

Get or set the remote user

=head1 See Also

Please also take a look at L<HTTP::Response>, L<HTTP::Request>,
L<POE::Component::Server::HTTP>, L<URI>, L<POE> and L<POE::Filter::HTTPD>.

=head1 TODO

=head1 AUTHOR

Arthur Bergman, arthur@contiller.se

Additional hacking by Philip Gwyn, poe-at-pied.nu

Released under the same terms as POE.

=cut
