use strict;
package POE::Component::Server::HTTP::Request;

use vars qw(@ISA);
use HTTP::Request;
@ISA = qw(HTTP::Request);

sub new {
    my $package = shift;
    my $self = $package->SUPER::new(@_);
    $self->is_error($self->method eq 'ERROR');
    return $self
}

sub connection {
    return $_[0]->{connection};
}

sub is_error {
    my $self = shift;
    if (@_) {
        if($_[0]) {
            $self->{is_error} = 1;
        } else {
            $self->{is_error} = 0;
        }
    }
    return $self->{is_error};
}

1;

__END__

=head1 NAME

POE::Component::Server::HTTP::Request - Request object for a POE HTTP Daemon

=head1 SYNOPSIS

    sub handler1 {
        my ($request, $response) = @_;
        my $uri = $request->uri;

        my $connection = $response->connection;

        unless( $request->is_error) {
            my $uniqueID = $connect->ID;
            $connection->on_close( 'conn_close', $uniqueID );
        }

    }

=head1 DESCRIPTION

POE::Component::Server::HTTP::Request encapsulates information related to
a single HTTP request.  Because of HTTP connection persistance, a single
connection may be use for more then one request and response.

POE::Component::Server::HTTP::Response is a sub-class of L<HTTP::Request>.
Use those methods to get information about a request.

=head1 METHODS

=head2 connection

Returns the connection object.  See
L<POE::Component::Server::HTTP::Connection>.

=head2 is_error

Get or set the error state on a request.


=head1 See Also

Please also take a look at L<HTTP::Response>, L<HTTP::Request>,
L<POE::Component::Server::HTTP>, L<URI>, L<POE> and L<POE::Filter::HTTPD>.

=head1 TODO

=head1 AUTHOR

Arthur Bergman, arthur@contiller.se

Additional hacking by Philip Gwyn, poe-at-pied.nu

Released under the same terms as POE.

=cut
