#!/usr/bin/perl -w

use strict;
use Test::More tests => 1 + 8*2;

use LWP::UserAgent;
use LWP::ConnCache;
use HTTP::Request;
use POE;
use POE::Component::Server::HTTP;
use Data::Dumper;

my $PORT = 2080;

my $pid = fork;
die "Unable to fork: $!" unless defined $pid;

END {
    if ($pid) {
        kill 2, $pid or warn "Unable to kill $pid: $!";
        my $kid = waitpid( $pid, 0 );
        if( $kid == $pid ) {
            is( $?, 0, "Server didn't have an error ($?)" );
        }
        else {
            fail( "Child already exited: $kid" );
        }
        
    }
}

my %CONNS;

####################################################################
if ($pid) {                      # we are parent
    # stop kernel from griping
    ${$poe_kernel->[POE::Kernel::KR_RUN]} |=
      POE::Kernel::KR_RUN_CALLED;

    print STDERR "$$: Sleep 2...";
    sleep 2;
    print STDERR "continue\n";

    my $UA = LWP::UserAgent->new;
  again:
    my $req=HTTP::Request->new(GET => "http://localhost:$PORT/");
    my $resp=$UA->request($req);

    ok($resp->is_success, "got index") or die "resp=", Dumper $resp;
    my $content=$resp->content;
    ok($content =~ /this is top/, "got top index");

    $req=HTTP::Request->new(GET => "http://localhost:$PORT/honk/something.html");
    $resp=$UA->request($req);

    ok($resp->is_success, "got something");
    $content=$resp->content;
    ok($content =~ /this is honk/, "something honked");

    $req=HTTP::Request->new(GET => "http://localhost:$PORT/bonk/zip.html");
    $resp=$UA->request($req);

    ok(($resp->is_success and $resp->content_type eq 'text/html'),
       "get text/html");
    $content=$resp->content;
    ok($content =~ /my friend/, 'my friend');

    $req=HTTP::Request->new(GET => "http://localhost:$PORT/bonk/");
    $resp=$UA->request($req);

    ok(($resp->is_success and $resp->content_type eq 'text/plain'),
                   "get text/plain");
    $content=$resp->content;
    ok($content =~ /this is bonk/, 'this is bonk');

    unless ($UA->conn_cache) {
        diag( "Enabling Keep-Alive and going again" );
        $UA->conn_cache( LWP::ConnCache->new() );
        goto again;
    }
    undef( $UA );
    print STDERR "$$: Sleep 2...";
    sleep 2;
    print STDERR "continue\n";
}

####################################################################
else {                          # we are the child
  $poe_kernel->has_forked if $poe_kernel->can( 'has_forked' );
  my $aliases = POE::Component::Server::HTTP->new(
     Port => $PORT,
     Address=>'localhost',
     MapOrder=>'bottom-first',
     ContentHandler => { '/' => \&top,
                         '/honk/' => \&honk,
                         '/bonk/' => \&bonk,
                         '/bonk/zip.html' => \&bonk2,
#                         '/shutdown.html' => \&shutdown
                       },
#     ErrorHandler => { '/' => \&error },
     Headers => { Server => 'TestServer' },
  );

  POE::Session->create(
        inline_states => {
            _start => \&app_start,
            on_close => \&app_on_close,
            bonk => \&app_bonk,
            sig_INT => \&app_INT,
        },
        heap => $aliases
    );

  $poe_kernel->run;
#  use Data::Dumper;
#  warn "$$: CONNECTIONS ", Dumper \%CONNS;
  die "We have active connections" if keys %CONNS;
}


#######################################
sub top
{
    my ($request, $response) = @_;
    $response->code(RC_OK);
    $response->content_type('text/plain');
    $response->content("this is top");
    return RC_OK;
}

#######################################
sub honk
{
    my ($request, $response) = @_;
    $response->code(RC_OK);
    $response->content_type('text/plain');
    $response->content("this is honk");
    return RC_OK;
}

#######################################
sub bonk
{
    my ($request, $response) = @_;
    $poe_kernel->post( APP => 'bonk', $request, $response );
    return RC_WAIT;
}

#######################################
sub bonk2
{
    my ($request, $response) = @_;
    $response->code(RC_OK);
    $response->content_type('text/html');
    $response->content(<<'    HTML');
<html>
<head><title>YEAH!</title></head>
<body><p>This, my friend, is the page you've been looking for.</p></body>
</html>
    HTML
    return RC_OK;
}


#######################################
sub app_start
{
    $_[KERNEL]->alias_set( 'APP' );
    $_[KERNEL]->sig( INT => 'sig_INT' );
}

#######################################
sub app_bonk
{
    my( $request, $response ) = @_[ ARG0, ARG1 ];

    my $c = $response->connection;
    $c->on_close( on_close => [ $c->ID, 'something' ], 'more' );
    $CONNS{ $c->ID }++;

    $response->code(RC_OK);
    $response->content_type('text/plain');
    $response->content("this is bonk");
    $response->continue;

    return;
}


#######################################
sub app_on_close
{
    my( $args, $more ) = @_[ARG0, ARG1];
    die "on_close without 2 arguments" unless $args and $more;
    die "First isn't the wheel ID=$args->[0]" unless $args->[0];
    die " ... with some extra data" unless $args->[1] eq 'something';
    die "Second isn't a string" unless $more eq 'more';
    $CONNS{ $args->[0] }--;
    delete $CONNS{ $args->[0] } if $CONNS{ $args->[0] } == 0;
}

#######################################
sub app_INT
{
    my( $kernel, $heap ) = @_[ KERNEL, HEAP ];
    $kernel->sig_handled;

#    $kernel->alias_remove( 'APP' );
    $kernel->call( $heap->{httpd}, 'shutdown' );
}

